<?php
if (!defined('ABSPATH')) {
  exit(); // Exit if accessed directly
}

// Import required files
require_once __DIR__ . '/woo-commerce.php';
require_once __DIR__ . '/rest.php';

// Utility class for managing IAP settings
class WebtonativeIAPUtil
{
  public static function getIAPSettings()
  {
    $defaults = array(
      'cart_button_text' => 'Buy Now',
      'processing_button_text' => 'Processing...',
      'failed_button_text' => 'Payment Failed',
      'payment_completed_button_text' => 'Payment Completed'
    );

    // return wp_parse_args(get_option('wtn_iap_settings'), $defaults);

    $saved_settings = get_option('wtn_iap_settings', array());

    // Explicitly merge saved settings with defaults
    foreach ($defaults as $key => $value) {
      if (!isset($saved_settings[$key]) || empty($saved_settings[$key])) {
        $saved_settings[$key] = $value;
      }
    }

    return $saved_settings;
  }

  public static function saveIAPSettings($settings)
  {
    update_option('wtn_iap_settings', $settings);
  }
}

// Main class for Webtonative IAP
class WebtonativeIAP
{
  public function __construct()
  {
    // Initialize REST controller
    new WebtonativeIAPRestController();

    // Add submenu for IAP settings
    add_action('admin_menu', array($this, 'add_submenu'));
  }

  // Add submenu for IAP settings
  public function add_submenu()
  {
    add_submenu_page(
      'webtonative-settings',             // Parent menu slug
      'WooCommerce IAP Settings',         // Page title
      'WooCommerce IAP',                  // Submenu title
      'manage_options',                   // Capability
      'wtn_woocom_iap_settings',          // Menu slug
      array($this, 'settings_page_html')  // Callback for page HTML
    );
  }

  // HTML for the IAP settings page
  public function settings_page_html()
  {
    // Check user capability
    if (!current_user_can('manage_options')) {
      wp_die(__('Sorry, you are not allowed to access this page.'));
    }

    $settings = WebtonativeIAPUtil::getIAPSettings();

    $cartButtonText = $settings['cart_button_text'];
    $processingButtonText = $settings['processing_button_text'];
    $failedButtonText = $settings['failed_button_text'];
    $paymentCompletedButtonText = $settings['payment_completed_button_text'];

    // Handle form submission
    if (
      $_SERVER['REQUEST_METHOD'] === 'POST' &&
      isset($_POST['iap_settings_nonce']) &&
      wp_verify_nonce($_POST['iap_settings_nonce'], 'wtn-iap-settings_nonce')
    ) {

      // Save settings
      $newSettings = array(
        'cart_button_text' => sanitize_text_field($_POST['cart_button_text']),
        'processing_button_text' => sanitize_text_field($_POST['processing_button_text']),
        'failed_button_text' => sanitize_text_field($_POST['failed_button_text']),
        'payment_completed_button_text' => sanitize_text_field($_POST['payment_completed_button_text']),
      );

      WebtonativeIAPUtil::saveIAPSettings($newSettings);
      $settings = $newSettings; // Update displayed values
      echo '<div class="updated"><p>Settings saved successfully!</p></div>';
    }
?>
    <div class="wrap">
      <h1>WooCommerce IAP Settings</h1>
      <form method="post">
        <?php wp_nonce_field('wtn-iap-settings_nonce', 'iap_settings_nonce'); ?>

        <table class="form-table">
          <tr>
            <th scope="row">
              <label for="cart_button_text">Cart Button Text</label>
            </th>
            <td>
              <input type="text" name="cart_button_text" id="cart_button_text"
                value="<?php echo esc_attr($cartButtonText); ?>" class="regular-text" />
            </td>
          </tr>
          <tr>
            <th scope="row">
              <label for="processing_button_text">Processing Button Text</label>
            </th>
            <td>
              <input type="text" name="processing_button_text" id="processing_button_text"
                value="<?php echo esc_attr($processingButtonText); ?>" class="regular-text" />
            </td>
          </tr>
          <tr>
            <th scope="row">
              <label for="failed_button_text">Failed Button Text</label>
            </th>
            <td>
              <input type="text" name="failed_button_text" id="failed_button_text"
                value="<?php echo esc_attr($failedButtonText); ?>" class="regular-text" />
            </td>
          </tr>
          <tr>
            <th scope="row">
              <label for="payment_completed_button_text">Payment Completed Button Text</label>
            </th>
            <td>
              <input type="text" name="payment_completed_button_text" id="payment_completed_button_text"
                value="<?php echo esc_attr($paymentCompletedButtonText); ?>" class="regular-text" />
            </td>
          </tr>
        </table>

        <?php submit_button('Save Settings'); ?>
      </form>
    </div>
<?php
  }
}

// Initialize the IAP plugin
add_action('plugins_loaded', function () {
  new WebtonativeIAP();
});
