(function () {
    // Ensure WTN.MediaPlayer exists
    if (typeof window.WTN === "undefined" || typeof window.WTN.MediaPlayer === "undefined") {
        console.error("WTN MediaPlayer is not available.");
        return;
    }

    // Check QTMediaSettings for enabled monitoring
    const audioMonitoringEnabled = QTMediaSettings.audioMonitoring === "yes";
    const videoMonitoringEnabled = QTMediaSettings.videoMonitoring === "yes";

    // Initialize global active media element tracker
    window.activeMediaElement = null;

    // Native media control functions
    function playMedia(url, image) {
        window.WTN.MediaPlayer.playMedia({ url, image });
    }

    function pauseMedia() {
        window.WTN.MediaPlayer.pauseMedia();
    }

    function stopMedia() {
        window.WTN.MediaPlayer.stopMedia();
    }

    // Function to bind media controls to <audio> and <video> elements
    function bindMediaElement(mediaElement) {
        mediaElement.addEventListener("play", () => {
            if (window.activeMediaElement && window.activeMediaElement !== mediaElement) {
                window.activeMediaElement.pause();
            }
            window.activeMediaElement = mediaElement;
            const mediaUrl = mediaElement.currentSrc;
            const mediaImage = mediaElement.getAttribute("data-thumbnail") || "";
            playMedia(mediaUrl, mediaImage);
        });

        mediaElement.addEventListener("pause", () => {
            if (window.activeMediaElement === mediaElement) {
                pauseMedia();
            }
        });

        mediaElement.addEventListener("ended", () => {
            if (window.activeMediaElement === mediaElement) {
                stopMedia();
                window.activeMediaElement = null;
            }
        });
    }

    // Initialize media controls for <audio> and <video> elements
    function initializeMediaControls() {
        if (audioMonitoringEnabled) {
            const audioElements = document.querySelectorAll("audio");
            audioElements.forEach((audioElement) => bindMediaElement(audioElement));
        }

        if (videoMonitoringEnabled) {
            const videoElements = document.querySelectorAll("video");
            videoElements.forEach((videoElement) => bindMediaElement(videoElement));
        }
    }

    // Monitor for dynamically added media elements
    function monitorDynamicMedia() {
        const container = document.body;
        const observer = new MutationObserver((mutations) => {
            mutations.forEach((mutation) => {
                mutation.addedNodes.forEach((node) => {
                    if (node.tagName === "AUDIO" && audioMonitoringEnabled) {
                        console.log("New audio element detected:", node);
                        bindMediaElement(node);
                    }

                    if (node.tagName === "VIDEO" && videoMonitoringEnabled) {
                        console.log("New video element detected:", node);
                        bindMediaElement(node);
                    }
                });
            });
        });

        observer.observe(container, {
            childList: true,
            subtree: true,
        });

        console.log("Dynamic media monitoring initialized.");
    }

    // Initialize functionality when DOM is fully loaded
    document.addEventListener("DOMContentLoaded", () => {
        initializeMediaControls();
        monitorDynamicMedia();
    });
})();
