<?php

/**
 * Plugin Name: Simple IAP
 * Description: A plugin to handle IAP purchases with verification.
 * Version: 1.2
 * Author: WebtoNative
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// ============================== Admin Settings Page ==============================

// Add settings page to WordPress admin
add_action('admin_menu', function () {
    add_submenu_page(
        'options-general.php',
        'IAP Settings',
        'WTN IAP (In-App Purchase)',
        'manage_options',
        'simple-iap-settings',
        'simple_iap_render_settings_page'
    );
});

// Register settings
add_action('admin_init', function () {
    // App Store Secret Key
    register_setting('simple_iap_settings', 'simple_iap_appstore_secret', [
        'type'              => 'string',
        'sanitize_callback' => 'sanitize_text_field',
        'default'           => '',
    ]);

    // Callback Function
    register_setting('simple_iap_settings', 'simple_iap_callback_function', [
        'type'              => 'string',
        'sanitize_callback' => 'sanitize_text_field',
        'default'           => '',
    ]);

    // Add settings section
    add_settings_section('simple_iap_settings_section', 'IAP Settings', null, 'simple-iap-settings');

    // Add fields
    add_settings_field('simple_iap_appstore_secret', 'App Store Secret Key', function () {
        $value = get_option('simple_iap_appstore_secret', '');
        echo '<input type="text" name="simple_iap_appstore_secret" value="' . esc_attr($value) . '" class="regular-text">';
        echo '<p class="description">Enter your App Store Connect shared secret key for iOS IAP verification.</p>';
    }, 'simple-iap-settings', 'simple_iap_settings_section');

    add_settings_field('simple_iap_callback_function', 'Callback Function Name', function () {
        $value = get_option('simple_iap_callback_function', '');
        echo '<input type="text" name="simple_iap_callback_function" value="' . esc_attr($value) . '" class="regular-text">';
        echo '<p class="description">Enter the function name to call after successful verification. Function must accept two parameters: $platform and $receipt_data.</p>';
    }, 'simple-iap-settings', 'simple_iap_settings_section');
});

// Render the settings page
function simple_iap_render_settings_page()
{
?>
    <div class="wrap">
        <h1>WTN IAP (In-App Purchase) Settings</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('simple_iap_settings');
            do_settings_sections('simple-iap-settings');
            submit_button();
            ?>
        </form>
    </div>
<?php
}

// ============================== Frontend Features ==============================

// Enqueue scripts
add_action('wp_enqueue_scripts', function () {
    wp_enqueue_script('simple-iap-js', plugin_dir_url(__FILE__) . 'js/wtn-iap.js', [], '1.1', true);

    wp_localize_script('simple-iap-js', 'simpleIAPData', [
        'ajax_url' => admin_url('admin-ajax.php'),
        'nonce'    => wp_create_nonce('simple_iap_nonce'),
        'isLoggedIn' => is_user_logged_in(),
        'iosSecretKey' => get_option('simple_iap_appstore_secret')
    ]);
});

// Shortcode for IAP button
add_shortcode('wtn_iap', function ($atts) {
    $atts = shortcode_atts([
        'google_play_id' => '',
        'app_store_id'   => '',
        'product_type'   => 'inapp',
        'is_consumable'  => 'false',
    ], $atts);

    ob_start();
?>
    <button id="iap-button" class="iap-button"
        data-google-play-id="<?php echo esc_attr($atts['google_play_id']); ?>"
        data-app-store-id="<?php echo esc_attr($atts['app_store_id']); ?>"
        data-product-type="<?php echo esc_attr($atts['product_type']); ?>"
        data-is-consumable="<?php echo esc_attr($atts['is_consumable']); ?>">
        Buy Now
    </button>
<?php
    return ob_get_clean();
});

// ============================== IAP Verification ==============================

// AJAX Handler for IAP Verification
add_action('wp_ajax_simple_iap_verify', 'simple_iap_verify');
add_action('wp_ajax_nopriv_simple_iap_verify', 'simple_iap_verify');

function simple_iap_verify()
{
    check_ajax_referer('simple_iap_nonce', 'nonce');

    $platform = sanitize_text_field($_POST['platform'] ?? '');
    $receipt  = sanitize_text_field($_POST['receipt'] ?? '');

    if (empty($platform) || empty($receipt)) {
        wp_send_json_error(['message' => 'Invalid data received.']);
        return;
    }

    try {
        $result = ($platform === 'ios') ? simple_iap_verify_ios_receipt($receipt) : simple_iap_verify_android_receipt($receipt);

        if ($result['success']) {
            $callback_function = get_option('simple_iap_callback_function');
            if (!empty($callback_function) && function_exists($callback_function)) {
                call_user_func($callback_function, $platform, ['receipt' => $receipt, 'verification_result' => $result]);
            }

            wp_send_json_success(['message' => 'Verification successful.']);
        } else {
            wp_send_json_error(['message' => $result['message']]);
        }
    } catch (Exception $e) {
        wp_send_json_error(['message' => $e->getMessage()]);
    }
}

// Function to verify iOS receipt
function simple_iap_verify_ios_receipt($receipt)
{
    $secret_key = get_option('simple_iap_appstore_secret');
    if (empty($secret_key)) {
        return ['success' => false, 'message' => 'App Store secret key not configured'];
    }

    // Define endpoints
    $production_endpoint = 'https://buy.itunes.apple.com/verifyReceipt';
    $sandbox_endpoint = 'https://sandbox.itunes.apple.com/verifyReceipt';

    // Create data payload
    $data = json_encode(['receipt-data' => $receipt, 'password' => $secret_key]);

    // First, try production endpoint
    $response = wp_remote_post($production_endpoint, [
        'headers' => ['Content-Type' => 'application/json'],
        'body'    => $data,
        'timeout' => 45,
    ]);

    $result = handle_ios_response($response, $sandbox_endpoint, $data);

    return $result;
}

// Helper function to handle iOS receipt response
function handle_ios_response($response, $sandbox_endpoint, $data)
{
    if (is_wp_error($response)) {
        return ['success' => false, 'message' => $response->get_error_message()];
    }

    $body = json_decode(wp_remote_retrieve_body($response), true);

    if (!isset($body['status'])) {
        return ['success' => false, 'message' => 'Invalid response from App Store.'];
    }

    // Status 21007 indicates a sandbox receipt was sent to production endpoint
    if ($body['status'] === 21007) {
        $sandbox_response = wp_remote_post($sandbox_endpoint, [
            'headers' => ['Content-Type' => 'application/json'],
            'body'    => $data,
            'timeout' => 45,
        ]);

        if (is_wp_error($sandbox_response)) {
            return ['success' => false, 'message' => $sandbox_response->get_error_message()];
        }

        $sandbox_body = json_decode(wp_remote_retrieve_body($sandbox_response), true);

        if ($sandbox_body['status'] === 0) {
            return [
                'success' => true,
                'message' => 'Sandbox receipt verified successfully.',
                'data'    => $sandbox_body
            ];
        } else {
            return [
                'success' => false,
                'message' => 'Sandbox receipt verification failed.',
                'data'    => $sandbox_body
            ];
        }
    }

    if ($body['status'] === 0) {
        return [
            'success' => true,
            'message' => 'Receipt verified successfully.',
            'data'    => $body
        ];
    } else {
        return [
            'success' => false,
            'message' => 'Invalid receipt data.',
            'data'    => $body
        ];
    }
}


// Placeholder for Android receipt verification
function simple_iap_verify_android_receipt($receipt)
{
    return ['success' => true, 'data' => $receipt]; // Implement actual Android verification here
}
