<?php

/**
 * Plugin Name: WebtoNative Biometric Authentication
 * Description: Add biometric authentication features to your WordPress app.
 * Version: 1.1.0
 * Author: webtonative
 */
if (!defined('ABSPATH')) exit;

// Enqueue Scripts
add_action('wp_enqueue_scripts', 'wtn_biometric_enqueue_scripts');
function wtn_biometric_enqueue_scripts()
{
    wp_enqueue_script('wtn-biometric-js', plugins_url('assets/biometric.js', __FILE__), ['jquery'], '1.0.0', true);

    wp_localize_script('wtn-biometric-js', 'WTN_Biometric_Settings', [
        'biometricEnabled' => (bool) get_option('wtn_biometric_enabled', false),
        'promptOnLogout' => (bool) get_option('wtn_biometric_prompt_on_logout', false),
        'promptOnResume' => (bool) get_option('wtn_biometric_prompt_on_resume', false),
        'promptOnOpen'   => (bool) get_option('wtn_biometric_prompt_on_open', false),
        'restApiUrl'     => esc_url_raw(rest_url('wtn/v1/biometric-login')),
        'isLoggedIn'     => is_user_logged_in(),
        'userSecret'     => is_user_logged_in() ? get_user_meta(get_current_user_id(), 'wtn_biometric_secret', true) : '',
        'wpNonce'        => wp_create_nonce('wp_rest'),
        'delayTime' => get_option('wtn_biometric_delay', 5),
    ]);
}

// Register REST API Routes
add_action('rest_api_init', function () {
    register_rest_route('wtn/v1', '/biometric-login', [
        'methods' => 'POST',
        'callback' => 'wtn_biometric_login_endpoint',
        'permission_callback' => '__return_true',
    ]);
});

add_action('rest_api_init', function () {
    register_rest_route('wtn/v1', '/session-status', [
        'methods'  => 'GET',
        'callback' => 'wtn_check_session_status',
        'permission_callback' => '__return_true', // Allow public access
    ]);
});

// handle forced logout event
add_action('wp_logout', function () {
    wp_redirect(home_url('?logged_out=true'));
    exit;
});

// REST API Callback
function wtn_biometric_login_endpoint(WP_REST_Request $request)
{
    $secret = sanitize_text_field($request->get_param('secret'));
    $action = $request->get_param('action'); // New parameter for login

    if (!is_user_logged_in() && $action !== 'login') {
        return new WP_Error('not_logged_in', 'User must be logged in', ['status' => 401]);
    }

    $user_id = get_current_user_id();

    if ($action === 'enable') {
        error_log("enableddddddddddd");
        update_user_meta($user_id, 'wtn_biometric_secret', $secret);
        return ['success' => true, 'message' => 'Biometric authentication enabled'];
    } elseif ($action === 'disable') {
        error_log("disableddddddddddd");
        delete_user_meta($user_id, 'wtn_biometric_secret');
        return ['success' => true, 'message' => 'Biometric authentication disabled'];
    } elseif ($action === 'login') {
        // Validate secret and log in the user
        $user = get_users([
            'meta_key' => 'wtn_biometric_secret',
            'meta_value' => $secret,
            'number' => 1,
        ]);

        if (!empty($user)) {
            wp_set_current_user($user[0]->ID);
            wp_set_auth_cookie($user[0]->ID, true);
            return ['success' => true, 'message' => 'User logged in', 'redirect_url' => home_url()];
        } else {
            return new WP_Error('invalid_secret', 'Invalid biometric secret', ['status' => 403]);
        }
    }

    return new WP_Error('invalid_action', 'Invalid action parameter', ['status' => 400]);
}

// Add Admin Menu for Settings
add_action('admin_menu', 'wtn_biometric_add_admin_menu');
function wtn_biometric_add_admin_menu()
{
    add_options_page(
        'Biometric Authentication Settings',
        'WTN Biometric Authentication',
        'manage_options',
        'wtn_biometric',
        'wtn_biometric_settings_page'
    );
}

// Register Settings
add_action('admin_init', 'wtn_biometric_register_settings');
function wtn_biometric_register_settings()
{
    register_setting('wtn_biometric_settings', 'wtn_biometric_enabled', [
        'type' => 'boolean',
        'sanitize_callback' => 'rest_sanitize_boolean',
        'default' => false,
    ]);
    register_setting('wtn_biometric_settings', 'wtn_biometric_prompt_on_logout');
    register_setting('wtn_biometric_settings', 'wtn_biometric_prompt_on_resume');
    register_setting('wtn_biometric_settings', 'wtn_biometric_prompt_on_open');
    register_setting('wtn_biometric_settings', 'wtn_biometric_delay', [
        'type' => 'integer',
        'sanitize_callback' => 'absint',
        'default' => 5, // Default to 5 minutes
    ]);
}

// Settings Page
function wtn_biometric_settings_page()
{
?>
    <div class="wrap">
        <h1>Webtonative Biometric Authentication Settings</h1>
        <form method="post" action="options.php">
            <?php settings_fields('wtn_biometric_settings'); ?>
            <?php do_settings_sections('wtn_biometric_settings'); ?>

            <table class="form-table">
                <tr valign="top">
                    <th scope="row">
                        <label for="wtn_biometric_enabled">Enable Biometrics</label>
                    </th>
                    <td>
                        <input type="checkbox" id="wtn_biometric_enabled" name="wtn_biometric_enabled" value="1" <?php checked(get_option('wtn_biometric_enabled', false)); ?>>
                        <p class="description">Check this box to enable biometric authentication for users.</p>
                    </td>
                </tr>
                <!-- <tr valign="top">
                    <th scope="row">Prompt on Logout</th>
                    <td>
                        <input type="checkbox" name="wtn_biometric_prompt_on_logout" value="1" <?php checked(1, get_option('wtn_biometric_prompt_on_logout', false)); ?>>
                        Enable biometric prompt on logout
                    </td>
                </tr> -->
                <tr valign="top">
                    <th scope="row">Prompt on Resume</th>
                    <td>
                        <input type="checkbox" name="wtn_biometric_prompt_on_resume" value="1" <?php checked(1, get_option('wtn_biometric_prompt_on_resume', false)); ?>>
                        Enable biometric prompt on app resume
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Prompt on Open</th>
                    <td>
                        <input type="checkbox" name="wtn_biometric_prompt_on_open" value="1" <?php checked(1, get_option('wtn_biometric_prompt_on_open', false)); ?>>
                        Enable biometric prompt on app open
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">
                        <label for="wtn_biometric_delay">Biometric Timeout (Minutes)</label>
                    </th>
                    <td>
                        <input type="number" id="wtn_biometric_delay" name="wtn_biometric_delay" value="<?php echo esc_attr(get_option('wtn_biometric_delay', 5)); ?>" min="1">
                        <p class="description">Enter the delay in minutes before requiring biometric authentication.</p>
                    </td>
                </tr>
            </table>

            <?php submit_button(); ?>
        </form>
    </div>
<?php
}

// Add Shortcode for User Biometric Settings
add_shortcode('wtn_biometric_settings', 'wtn_biometric_settings_shortcode');
function wtn_biometric_settings_shortcode()
{
    if (!is_user_logged_in()) {
        return;
    }

    $user_id = get_current_user_id();
    $is_biometric_enabled = get_user_meta($user_id, 'wtn_biometric_secret', true);

    ob_start();
?>
    <div id="wtn-biometric-settings" style="display: none;">
        <h3>Biometric Authentication</h3>
        <label>
            <input type="checkbox" id="enable-biometric-auth" <?php checked(!empty($is_biometric_enabled)); ?>>
            Enable Biometric Authentication
        </label>
        <p id="biometric-status-message"></p>
    </div>
    <script>
        document.addEventListener('DOMContentLoaded', () => {
            const checkbox = document.getElementById('enable-biometric-auth');
            const statusMessage = document.getElementById('biometric-status-message');

            // Update Biometric Settings via REST API
            function updateBiometricSettings(enabled, secret) {
                fetch(WTN_Biometric_Settings.restApiUrl, {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                            'X-WP-Nonce': WTN_Biometric_Settings.wpNonce
                        },
                        body: JSON.stringify({
                            action: enabled ? 'enable' : 'disable',
                            secret: secret
                        })
                    })
                    .then(response => response.json())
                    .then(data => {
                        if (data.success) {
                            statusMessage.textContent = enabled ?
                                "Biometric authentication enabled successfully." :
                                "Biometric authentication disabled.";
                        } else {
                            checkbox.checked = !enabled;
                            statusMessage.textContent = "Failed to update biometric settings.";
                        }
                    })
                    .catch(() => {
                        checkbox.checked = !enabled;
                        statusMessage.textContent = "An error occurred. Please try again.";
                    });
            }

            // Handle Biometric Checkbox Change
            checkbox.addEventListener('change', () => {
                if (checkbox.checked) {
                    // Enable biometric authentication
                    window.WTN.Biometric.show({
                        prompt: "Authenticate to enable biometric authentication.",
                        callback: function(data) {
                            if (data.isSuccess) {
                                const secret = data.secret;
                                updateBiometricSettings(true, secret);
                            } else {
                                checkbox.checked = false;
                                statusMessage.textContent = "Biometric authentication failed.";
                            }
                        }
                    });
                } else {
                    // Disable biometric authentication
                    updateBiometricSettings(false, null);
                }
            });

        });
    </script>
<?php
    return ob_get_clean();
}

// handle biometric login button UI
add_shortcode('wtn_biometric_login', 'wtn_biometric_login_shortcode');
function wtn_biometric_login_shortcode()
{
    if (is_user_logged_in()) {
        return;
    }

    // $user_id = get_current_user_id();
    // $is_biometric_enabled = get_user_meta($user_id, 'wtn_biometric_secret', true);

    // if (empty($is_biometric_enabled)) {
    //     return;
    // }

    ob_start();
?>
    <div id="wtn-biometric-login" style="display: none;">
        <button id="biometric-login-btn">Login with Biometric</button>
        <p id="biometric-login-status"></p>
    </div>
    <script>
        document.addEventListener('DOMContentLoaded', () => {
            const loginButton = document.getElementById('biometric-login-btn');
            const statusMessage = document.getElementById('biometric-login-status');

            // Biometric Login Button Click
            loginButton.addEventListener('click', () => {
                window.WTN.Biometric.show({
                    prompt: "Authenticate to login",
                    callback: function(data) {
                        if (data.isSuccess) {
                            biometricLogin(data.secret);
                        } else {
                            statusMessage.textContent = "Biometric authentication failed.";
                        }
                    }
                });
            });

            // Perform Login via REST API
            function biometricLogin(secret) {
                fetch(WTN_Biometric_Settings.restApiUrl, {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                            'X-WP-Nonce': WTN_Biometric_Settings.wpNonce
                        },
                        body: JSON.stringify({
                            action: 'login',
                            secret: secret
                        })
                    })
                    .then(response => response.json())
                    .then(data => {
                        if (data.success) {
                            window.location.href = data.redirect_url;
                        } else {
                            statusMessage.textContent = "Login failed. Invalid biometric credentials.";
                        }
                    })
                    .catch(() => {
                        statusMessage.textContent = "An error occurred during login.";
                    });
            }
        });
    </script>
<?php
    return ob_get_clean();
}
